
import io
import logging
import pytest

from traitlets import default
from .mockextension import MockExtensionApp
from nbclassic import shim


@pytest.fixture
def extapp_log():
    """An io stream with the NotebookApp's logging output"""
    stream = io.StringIO()
    return stream


@pytest.fixture(autouse=True)
def extapp_logcapture(monkeypatch, extapp_log):
    """"""
    @default('log')
    def _log_default(self):
        """Start logging for this application.
        The default is to log to stderr using a StreamHandler, if no default
        handler already exists.  The log level starts at logging.WARN, but this
        can be adjusted by setting the ``log_level`` attribute.
        """
        log = super(self.__class__, self)._log_default()
        _log_handler = logging.StreamHandler(extapp_log)
        _log_formatter = self._log_formatter_cls(
            fmt=self.log_format,
            datefmt=self.log_datefmt
        )
        _log_handler.setFormatter(_log_formatter)
        log.addHandler(_log_handler)
        return log

    monkeypatch.setattr(MockExtensionApp, '_log_default', _log_default)
    return _log_default


@pytest.fixture
def jp_server_config():
    return {
        "ServerApp": {
            "jpserver_extensions": {
                "nbclassic": True,
                "nbclassic.tests.shim.mockextension": True
            }
        }
    }


@pytest.fixture
def extensionapp(jp_serverapp):
    return jp_serverapp.extension_manager.extension_points["mockextension"].app


def list_test_params(param_input):
    """"""
    params = []
    for test in param_input:
        name, value = test[0], test[1]
        option = (
            '--MockExtensionApp.'
            '{name}={value}'
            .format(name=name, value=value)
        )
        params.append([[option], name, value])
    return params


@pytest.mark.parametrize(
    'jp_argv,trait_name,trait_value',
    list_test_params([
        ('enable_mathjax', False)
    ])
)
def test_EXTAPP_AND_NBAPP_SHIM_MSG(
    extensionapp,
    extapp_log,
    jp_argv,
    trait_name,
    trait_value
):
    log = extapp_log.getvalue()
    # Verify a shim warning appeared.
    log_msg = shim.EXTAPP_AND_NBAPP_SHIM_MSG(trait_name, 'MockExtensionApp')
    assert log_msg in log
    # Verify the trait was updated.
    assert getattr(extensionapp, trait_name) == trait_value


@pytest.mark.parametrize(
    'jp_argv,trait_name,trait_value',
    list_test_params([
        ('allow_origin', ''),
        ('allow_origin_pat', ''),
    ])
)
def test_EXTAPP_AND_SVAPP_SHIM_MSG(
    extensionapp,
    extapp_log,
    jp_argv,
    trait_name,
    trait_value
):
    log = extapp_log.getvalue()
    # Verify a shim warning appeared.
    log_msg = shim.EXTAPP_AND_SVAPP_SHIM_MSG(trait_name, 'MockExtensionApp')
    assert log_msg in log
    # Verify the trait was updated.
    assert getattr(extensionapp, trait_name) == trait_value


@pytest.mark.parametrize(
    'jp_argv,trait_name,trait_value',
    list_test_params([
        ('jinja_environment_options', {}),
        ('jinja_template_vars', {}),
        ('extra_template_paths', []),
        ('quit_button', True),
    ])
)
def test_NOT_EXTAPP_NBAPP_AND_SVAPP_SHIM_MSG(
    extensionapp,
    extapp_log,
    jp_argv,
    trait_name,
    trait_value
):
    log = extapp_log.getvalue()
    # Verify a shim warning appeared.
    log_msg = shim.NOT_EXTAPP_NBAPP_AND_SVAPP_SHIM_MSG(trait_name, 'MockExtensionApp')
    assert log_msg in log
    # Verify the trait was updated.
    assert getattr(extensionapp.serverapp, trait_name) == trait_value


@pytest.mark.parametrize(
    'jp_argv,trait_name,trait_value',
    list_test_params([
        ('allow_credentials', False),
    ])
)
def test_EXTAPP_TO_SVAPP_SHIM_MSG(
    extensionapp,
    extapp_log,
    jp_argv,
    trait_name,
    trait_value
):
    log = extapp_log.getvalue()
    # Verify a shim warning appeared.
    log_msg = shim.EXTAPP_TO_SVAPP_SHIM_MSG(trait_name, 'MockExtensionApp')
    assert log_msg in log
    # Verify the trait was updated.
    assert getattr(extensionapp.serverapp, trait_name) == trait_value


@pytest.mark.parametrize(
    'jp_argv,trait_name,trait_value',
    list_test_params([
        ('mathjax_config', 'TEST'),
        ('mathjax_url', 'TEST')
    ])
)
def test_EXTAPP_TO_NBAPP_SHIM_MSG(
    extensionapp,
    extapp_log,
    jp_argv,
    trait_name,
    trait_value
):
    log = extapp_log.getvalue()
    # Verify a shim warning appeared.
    log_msg = shim.EXTAPP_TO_NBAPP_SHIM_MSG(trait_name, 'MockExtensionApp')
    assert log_msg in log



import pytest
from nbclassic import shim


@pytest.fixture
def jp_server_config():
    return {
        "ServerApp": {
            "jpserver_extensions": {
                "nbclassic": True,
            }
        }
    }



@pytest.fixture
def nbclassic(jp_serverapp):
    return jp_serverapp.extension_manager.extension_points["notebook"].app


def list_test_params(param_input):
    params = []
    for test in param_input:
        name, value = test[0], test[1]
        option = (
            '--NotebookApp.'
            '{name}={value}'
            .format(name=name, value=value)
        )
        params.append([[option], name, value])
    return params


@pytest.mark.parametrize(
    'jp_argv,trait_name,trait_value',
    list_test_params([
        ('jinja_environment_options', {}),
        ('jinja_template_vars', {}),
        ('extra_template_paths', []),
        ('quit_button', True),
    ])
)
def test_NBAPP_AND_SVAPP_SHIM_MSG(
    nbclassic,
    nbapp_log,
    jp_argv,
    trait_name,
    trait_value
):
    log = nbapp_log.getvalue()
    # Verify a shim warning appeared.
    assert shim.NBAPP_AND_SVAPP_SHIM_MSG(trait_name) in log
    # Verify the trait was updated.
    assert getattr(nbclassic, trait_name) == trait_value


@pytest.mark.parametrize(
    'jp_argv,trait_name,trait_value',
    list_test_params([
        ('allow_origin', ''),
        ('allow_origin_pat', ''),
        ('allow_credentials', False),
        # ('allow_root', False)         # This trait is hardcoded by jupyter-server pytest extension.
    ])
)
def test_NBAPP_TO_SVAPP_SHIM_MSG(
    jp_serverapp,
    nbapp_log,
    jp_argv,
    trait_name,
    trait_value
):
    # Expected log message.
    log = nbapp_log.getvalue()
    expected_msg = shim.NBAPP_TO_SVAPP_SHIM_MSG(trait_name)
    # Verify a warning message was raised
    assert expected_msg in log
    # Verify that trait changed.
    assert getattr(jp_serverapp, trait_name) == trait_value


@pytest.mark.parametrize(
    'jp_argv,trait_name,trait_value',
    list_test_params([
        #('ignore_minified_js', True),
        ('enable_mathjax', False),
        ('mathjax_config', 'TEST'),
        ('mathjax_url', 'TEST')
    ])
)
def test_nbclassic_traits_pass_shim(
    nbclassic,
    jp_argv,
    trait_name,
    trait_value
):
    assert getattr(nbclassic, trait_name) == trait_value




import pytest


def test_classic_notebook_templates(jp_serverapp):
    classic_notebook_templates = [
        "notebook.html",
        "tree.html"
    ]
    # Get the server's template environment.
    template_env = jp_serverapp.web_app.settings.get("notebook_jinja2_env")

    for name in classic_notebook_templates:
        template_env.get_template(name)


async def test_classic_notebook_asset_URLS(jp_fetch):
    classic_notebook_paths = [
        # Some classic notebook asset paths
        '/static/notebook/js/main.js',
        '/static/services/contents.js',
        # NBclassic asset paths work too.
        '/static/notebook/notebook/js/main.js',
        '/static/notebook/services/contents.js',
    ]

    for url_path in classic_notebook_paths:
        r = await jp_fetch(url_path)
        assert r.code == 200



"""Basic tests for the notebook handlers.
"""

import pytest


@pytest.fixture
def notebooks(jp_create_notebook):
    nbpaths = (
        'notebook1.ipynb',
        'nbclassic_test_notebooks/notebook2.ipynb',
        'nbclassic_test_notebooks/level2/notebook3.ipynb'
    )
    for nb in nbpaths:
        jp_create_notebook(nb)
    return nbpaths


async def test_tree_handler(notebooks, jp_fetch):
    r = await jp_fetch('tree', 'nbclassic_test_notebooks')
    assert r.code == 200

    # Check that the tree template is loaded
    html = r.body.decode()
    assert "Files" in html
    assert "Running" in html
    assert "Clusters" in html


async def test_notebook_handler(notebooks, jp_fetch):
    for nbpath in notebooks:
        r = await jp_fetch('notebooks', nbpath)
        assert r.code == 200
        # Check that the notebook template is loaded
        html = r.body.decode()
        assert "Menu" in html
        assert "Kernel" in html
        assert nbpath in html


async def test_terminal_handler(jp_fetch):
        r = await jp_fetch('terminals', "1")
        assert r.code == 200
        # Check that the terminals template is loaded
        html = r.body.decode()
        assert "terminal-app" in html



def _jupyter_server_extension_paths():
    # Locally import to avoid install errors.
    from .notebookapp import NotebookApp

    return [
        {
            'module': 'nbclassic.notebookapp',
            'app': NotebookApp,
            'name': 'jupyter-nbclassic'
        },
        {
            'module': 'nbclassic.nbserver',
        }
    ]



import io
import logging
import pytest
from traitlets import default
from nbclassic.notebookapp import NotebookApp

pytest_plugins = ["jupyter_server.pytest_plugin"]


@pytest.fixture
def nbapp_log():
    """An io stream with the NotebookApp's logging output"""
    stream = io.StringIO()
    return stream


@pytest.fixture(autouse=True)
def notebookapp_logcapture(monkeypatch, nbapp_log):
    """"""
    @default('log')
    def _log_default(self):
        """Start logging for this application.
        The default is to log to stderr using a StreamHandler, if no default
        handler already exists.  The log level starts at logging.WARN, but this
        can be adjusted by setting the ``log_level`` attribute.
        """
        log = super(self.__class__, self)._log_default()
        _log_handler = logging.StreamHandler(nbapp_log)
        _log_formatter = self._log_formatter_cls(
            fmt=self.log_format,
            datefmt=self.log_datefmt
        )
        _log_handler.setFormatter(_log_formatter)
        log.addHandler(_log_handler)
        return log

    monkeypatch.setattr(NotebookApp, '_log_default', _log_default)
    return _log_default


@pytest.fixture
def jp_server_config():
    return {
        "ServerApp": {
            "jpserver_extensions": {
                "nbclassic": True
            }
        }
    }
